@@ -1,24 +1,21 @@
 package DBIx::Class::Admin;
 
-use warnings;
-use strict;
-
 # check deps
 BEGIN {
-  require DBIx::Class::Optional::Dependencies;
-  if (my $missing = DBIx::Class::Optional::Dependencies->req_missing_for ('admin') ) {
-    die "The following extra modules are required for DBIx::Class::Admin: $missing\n";
-  }
+  use Carp::Clan qw/^DBIx::Class/;
+  use DBIx::Class;
+  croak('The following modules are required for DBIx::Class::Admin ' . DBIx::Class::Optional::Dependencies->req_missing_for ('admin') )
+    unless DBIx::Class::Optional::Dependencies->req_ok_for ('admin');
 }
 
-use JSON::Any qw(DWIW PP JSON CPANEL XS);
 use Moose;
 use MooseX::Types::Moose qw/Int Str Any Bool/;
 use DBIx::Class::Admin::Types qw/DBICConnectInfo DBICHashRef/;
 use MooseX::Types::JSON qw(JSON);
 use MooseX::Types::Path::Class qw(Dir File);
-use MooseX::Types::LoadableClass qw(LoadableClass);
-use namespace::clean;
+use Try::Tiny;
+use JSON::Any qw(DWIW XS JSON);
+use namespace::autoclean;
 
 =head1 NAME
 
@@ -72,7 +69,7 @@ the class of the schema to load
 
 has 'schema_class' => (
   is  => 'ro',
-  isa => LoadableClass,
+  isa => Str,
 );
 
 
@@ -90,11 +87,14 @@ has 'schema' => (
 
 sub _build_schema {
   my ($self)  = @_;
+  require Class::MOP;
+  Class::MOP::load_class($self->schema_class);
 
-  $self->connect_info->[3]{ignore_version} = 1;
-  return $self->schema_class->clone->connection(@{$self->connect_info});
+  $self->connect_info->[3]->{ignore_version} =1;
+  return $self->schema_class->connect(@{$self->connect_info()} ); # ,  $self->connect_info->[3], { ignore_version => 1} );
 }
 
+
 =head2 resultset
 
 a resultset from the schema to operate on
@@ -135,7 +135,7 @@ has 'set' => (
 
 =head2 attrs
 
-a hash ref or json string to be used for passing additional info to the ->search call
+a hash ref or json string to be used for passing additonal info to the ->search call
 
 =cut
 
@@ -169,7 +169,7 @@ sub _build_connect_info {
 
 config_file provide a config_file to read connect_info from, if this is provided
 config_stanze should also be provided to locate where the connect_info is in the config
-The config file should be in a format readable by Config::Any.
+The config file should be in a format readable by Config::General
 
 =cut
 
@@ -182,7 +182,7 @@ has config_file => (
 
 =head2 config_stanza
 
-config_stanza for use with config_file should be a '::' delimited 'path' to the connection information
+config_stanza for use with config_file should be a '::' deliminated 'path' to the connection information
 designed for use with catalyst config files
 
 =cut
@@ -195,7 +195,7 @@ has 'config_stanza' => (
 
 =head2 config
 
-Instead of loading from a file the configuration can be provided directly as a hash ref.  Please note
+Instead of loading from a file the configuration can be provided directly as a hash ref.  Please note 
 config_stanza will still be required.
 
 =cut
@@ -209,6 +209,9 @@ has config => (
 sub _build_config {
   my ($self) = @_;
 
+  eval { require Config::Any }
+    or die ("Config::Any is required to parse the config file.\n");
+
   my $cfg = Config::Any->load_files ( {files => [$self->config_file], use_ext =>1, flatten_to_hash=>1});
 
   # just grab the config from the config file
@@ -230,17 +233,6 @@ has 'sql_dir' => (
 );
 
 
-=head2 sql_type
-
-The type of sql dialect to use for creating sql files from schema
-
-=cut
-
-has 'sql_type' => (
-  is     => 'ro',
-  isa    => Str,
-);
-
 =head2 version
 
 Used for install, the version which will be 'installed' in the schema
@@ -255,7 +247,7 @@ has version => (
 
 =head2 preversion
 
-Previous version of the schema to create an upgrade diff for, the full sql for that version of the sql must be in the sql_dir
+Previouse version of the schema to create an upgrade diff for, the full sql for that version of the sql must be in the sql_dir
 
 =cut
 
@@ -294,24 +286,6 @@ has '_confirm' => (
 );
 
 
-=head2 trace
-
-Toggle DBIx::Class debug output
-
-=cut
-
-has trace => (
-    is => 'rw',
-    isa => Bool,
-    trigger => \&_trigger_trace,
-);
-
-sub _trigger_trace {
-    my ($self, $new, $old) = @_;
-    $self->schema->storage->debug($new);
-}
-
-
 =head1 METHODS
 
 =head2 create
@@ -322,9 +296,8 @@ sub _trigger_trace {
 
 =back
 
-C<create> will generate sql for the supplied schema_class in sql_dir. The
-flavour of sql to generate can be controlled by supplying a sqlt_type which
-should be a L<SQL::Translator> name.
+L<create> will generate sql for the supplied schema_class in sql_dir.  The flavour of sql to 
+generate can be controlled by suppling a sqlt_type which should be a L<SQL::Translator> name.  
 
 Arguments for L<SQL::Translator> can be supplied in the sqlt_args hashref.
 
@@ -336,17 +309,12 @@ sub create {
   my ($self, $sqlt_type, $sqlt_args, $preversion) = @_;
 
   $preversion ||= $self->preversion();
-  $sqlt_type ||= $self->sql_type();
 
   my $schema = $self->schema();
+  # create the dir if does not exist
+  $self->sql_dir->mkpath() if ( ! -d $self->sql_dir);
 
-  $schema->create_ddl_dir(
-    $sqlt_type,
-    (defined $schema->schema_version ? $schema->schema_version : ""),
-    $self->sql_dir,
-    $preversion,
-    $sqlt_args,
-  );
+  $schema->create_ddl_dir( $sqlt_type, (defined $schema->schema_version ? $schema->schema_version : ""), $self->sql_dir->stringify, $preversion, $sqlt_args );
 }
 
 
@@ -366,12 +334,10 @@ B<MAKE SURE YOU BACKUP YOUR DB FIRST>
 sub upgrade {
   my ($self) = @_;
   my $schema = $self->schema();
-
   if (!$schema->get_db_version()) {
     # schema is unversioned
     $schema->throw_exception ("Could not determin current schema version, please either install() or deploy().\n");
   } else {
-    $schema->upgrade_directory ($self->sql_dir) if $self->sql_dir;  # this will override whatever default the schema has
     my $ret = $schema->upgrade();
     return $ret;
   }
@@ -386,9 +352,9 @@ sub upgrade {
 
 =back
 
-install is here to help when you want to move to L<DBIx::Class::Schema::Versioned> and have an existing
-database.  install will take a version and add the version tracking tables and 'install' the version.  No
-further ddl modification takes place.  Setting the force attribute to a true value will allow overriding of
+install is here to help when you want to move to L<DBIx::Class::Schema::Versioned> and have an existing 
+database.  install will take a version and add the version tracking tables and 'install' the version.  No 
+further ddl modification takes place.  Setting the force attribute to a true value will allow overriding of 
 already versioned databases.
 
 =cut
@@ -400,12 +366,12 @@ sub install {
   $version ||= $self->version();
   if (!$schema->get_db_version() ) {
     # schema is unversioned
-    print "Going to install schema version\n" if (!$self->quiet);
+    print "Going to install schema version\n";
     my $ret = $schema->install($version);
-    print "return is $ret\n" if (!$self->quiet);
+    print "retun is $ret\n";
   }
   elsif ($schema->get_db_version() and $self->force ) {
-    warn "Forcing install may not be a good idea\n";
+    carp "Forcing install may not be a good idea";
     if($self->_confirm() ) {
       $self->schema->_set_db_version({ version => $version});
     }
@@ -425,7 +391,7 @@ sub install {
 
 =back
 
-deploy will create the schema at the connected database.  C<$args> are passed straight to
+deploy will create the schema at the connected database.  C<$args> are passed straight to 
 L<DBIx::Class::Schema/deploy>.
 
 =cut
@@ -433,7 +399,13 @@ L<DBIx::Class::Schema/deploy>.
 sub deploy {
   my ($self, $args) = @_;
   my $schema = $self->schema();
-  $schema->deploy( $args, $self->sql_dir );
+  if (!$schema->get_db_version() ) {
+    # schema is unversioned
+    $schema->deploy( $args, $self->sql_dir)
+      or $schema->throw_exception ("Could not deploy schema.\n"); # FIXME deploy() does not return 1/0 on success/fail
+  } else {
+    $schema->throw_exception("A versioned schema has already been deployed, try upgrade instead.\n");
+  }
 }
 
 =head2 insert
@@ -455,7 +427,7 @@ sub insert {
   $rs ||= $self->resultset();
   $set ||= $self->set();
   my $resultset = $self->schema->resultset($rs);
-  my $obj = $resultset->new_result($set)->insert;
+  my $obj = $resultset->create( $set );
   print ''.ref($resultset).' ID: '.join(',',$obj->id())."\n" if (!$self->quiet);
 }
 
@@ -480,8 +452,7 @@ sub update {
   $where ||= $self->where();
   $set ||= $self->set();
   my $resultset = $self->schema->resultset($rs);
-  $resultset = $resultset->search_rs( $where )
-    if $where;
+  $resultset = $resultset->search( ($where||{}) );
 
   my $count = $resultset->count();
   print "This action will modify $count ".ref($resultset)." records.\n" if (!$self->quiet);
@@ -512,8 +483,7 @@ sub delete {
   $where ||= $self->where();
   $attrs ||= $self->attrs();
   my $resultset = $self->schema->resultset($rs);
-  $resultset = $resultset->search_rs( ($where||{}), ($attrs||()) )
-    if $where or $attrs;
+  $resultset = $resultset->search( ($where||{}), ($attrs||()) );
 
   my $count = $resultset->count();
   print "This action will delete $count ".ref($resultset)." records.\n" if (!$self->quiet);
@@ -532,7 +502,7 @@ sub delete {
 
 =back
 
-select takes the name of a resultset from the schema_class, a where hashref and a attrs to pass to ->search.
+select takes the name of a resultset from the schema_class, a where hashref and a attrs to pass to ->search. 
 The found data is returned in a array ref where the first row will be the columns list.
 
 =cut
@@ -544,12 +514,11 @@ sub select {
   $where ||= $self->where();
   $attrs ||= $self->attrs();
   my $resultset = $self->schema->resultset($rs);
-  $resultset = $resultset->search_rs( ($where||{}), ($attrs||()) )
-    if $where or $attrs;
+  $resultset = $resultset->search( ($where||{}), ($attrs||()) );
 
   my @data;
   my @columns = $resultset->result_source->columns();
-  push @data, [@columns];#
+  push @data, [@columns];# 
 
   while (my $row = $resultset->next()) {
     my @fields;
@@ -564,14 +533,12 @@ sub select {
 
 sub _confirm {
   my ($self) = @_;
-
+  print "Are you sure you want to do this? (type YES to confirm) \n";
   # mainly here for testing
   return 1 if ($self->meta->get_attribute('_confirm')->get_value($self));
-
-  print "Are you sure you want to do this? (type YES to confirm) \n";
   my $response = <STDIN>;
-
-  return ($response=~/^YES/);
+  return 1 if ($response=~/^YES/);
+  return;
 }
 
 sub _find_stanza {
@@ -585,20 +552,16 @@ sub _find_stanza {
       die ("Could not find $stanza in config, $path does not seem to exist.\n");
     }
   }
-  $cfg = $cfg->{connect_info} if exists $cfg->{connect_info};
   return $cfg;
 }
 
-=head1 FURTHER QUESTIONS?
+=head1 AUTHOR
 
-Check the list of L<additional DBIC resources|DBIx::Class/GETTING HELP/SUPPORT>.
+See L<DBIx::Class/CONTRIBUTORS>.
 
-=head1 COPYRIGHT AND LICENSE
+=head1 LICENSE
 
-This module is free software L<copyright|DBIx::Class/COPYRIGHT AND LICENSE>
-by the L<DBIx::Class (DBIC) authors|DBIx::Class/AUTHORS>. You can
-redistribute it and/or modify it under the same terms as the
-L<DBIx::Class library|DBIx::Class/COPYRIGHT AND LICENSE>.
+You may distribute this code under the same terms as Perl itself
 
 =cut
 
